<?php

/**
 * RemstoItem
 * Base class for processing remsto data items
 *
 * @author Ian Burnup
 */
class RemstoItem {

  protected $parent_type, $parent_id;

  public function __construct($remsto_data) {

    if ($remsto_data) {
      foreach ($remsto_data as $property => &$value) {
        if (property_exists($this, $property)) {
          $this->$property = &$value;
        }
      }
    }
  }
  
  /**
   * Truncate text
   * @param type $text
   * @param type $len
   */
  public function truncateText($text, $len) {

    if (!empty($len)) {
      $output = RemstoModuleFunctions::truncate_utf8(sanitize_text_field($text), $len, true, true);
    }
    else {
      $output = sanitize_text_field($text);
    }
    
    return $output;
  }

  /**
   * Format text item
   * @param type $variables
   * @return string
   */
  protected function formatTextValue($variables) {

    if (!empty($variables['value'])) {
      // Truncate text if value given
      if (isset($variables['truncate']) && $variables['truncate'] > 0) {
        $value = sanitize_text_field(RemstoModuleFunctions::truncate_utf8($variables['value'], $variables['truncate'], true, true));
      }
      else {
        $value = sanitize_text_field($variables['value']);
      }
      $label = $variables['label'];
      //$value = sanitize_text_field($variables['value']);
      //$class = $variables['class'];
      if (isset($variables['class'])) {
        $class = $variables['class'];
      }
      else {
        $class = '';
      }

      if (isset($variables['title'])) {
        $title = $variables['title'];
      }
      else {
        $title = '';
      }
      $output = "<div class='tpp-field $class' title='$title'>";
      if ($label !== '') {
        $output .= "<span class='tpp-label'>$label</span>";
      }
      $output .= "<span class='tpp-value'>$value</span></div>";
    }
    else {
      $output = '';
    }
    return $output;
  }

  /**
   * Format web site link
   * @param string $value
   * @return type
   */
  protected function formatWebsite($value, $icon = true) {

    if (!empty($value)) {

      $http = preg_match('/^http[^s]/i', $value);
      if (!$http) {
        $value = "http://" . $value;
      }

      $link = "<a href='" . $value . "' target='_blank' >Visit Website</a>";
      if ($icon) {
        $display_link = $this->formatIconTextValue(array(
          'icon_class' => 'icon-website',
          'value' => $link,
        ));
      }
      else {
        $display_link = $link;
      }

      return $display_link;
    }
  }

  
  /**
   * Format text value with an icon
   * @param type $variables
   * @return string
   */
  private function formatIconTextValue($variables) {

    $icon_class = $variables['icon_class'];
    if (isset($variables['title'])) {
      $title = $variables['title'];
    }
    else {
      $title = '';
    }

    $output = "<div class='$icon_class' title='$title'>";
    $output .= $variables['value'];
    $output .= '</div>';

    return $output;
  }

  /**
   * Format name
   * @param type $variables
   * @return type
   */
  public function formatNameDisplay($variables) {
    if ($variables['anon'] == 1) {
      $output = 'Client details withheld';
    }
    else {
      $output = $variables['first_name'] . ' ' . $variables['last_name'];
      if (isset($variables['town']) && $variables['town'] !== '') {
        $output .= ' ' . $variables['town'];
      }
    }
    return $output;
  }

  /**
   * Show client name
   * @param type $variables
   * @return type
   */
  protected function formatClientDisplay($variables) {

    if ($variables['anon'] == 1) {
      $output = 'Client details withheld';
    }
    elseif (!empty($variables['website'])) {
      $link_name = (!empty($variables['company_name'])) ? $variables['company_name'] : $variables['website'];
      $output = "<a class='external-link' href='http://{$variables['website']}' target='_blank'>{$link_name}</a>";
    }
    else {
      $output = $variables['company_name'];
    }
    return $output;
  }

  /**
   * Parse video URL.
   * @param type $video_link
   * @return type
   */
  protected function formatVideoLink($video_link) {

    $matches = array();
    if (preg_match(
            "#(?<=v=)[a-zA-Z0-9-]+(?=&)|(?<=v\/)[^&\n]+|(?<=v=)[^&\n]+|(?<=youtu.be/)[^&\n]+#", $video_link, $matches)) {
      $embed_code = "<iframe width='320' height='240' src='https://www.youtube.com/embed/" . $matches[0] . "' frameborder='0' allowfullscreen></iframe>";
    }
    else if (preg_match(
            "/^.*(vimeo\.com\/)((channels\/[A-z]+\/)|(groups\/[A-z]+\/videos\/))?([0-9]+)/", $video_link, $matches)) {
      $embed_code = "<iframe src='//player.vimeo.com/video/" . $matches[5] . "' width='320' height='240' frameborder='0' webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>";
    }
    else {
      $embed_code = "<video width='320' height='240' controls>";
      $embed_code .= "<source src='$video_link' type='video/mp4'></video>";
    }

    
    $output = "<div class='tpp-video-testimonial' id='tpp-video-testimonial-$this->id' data-id='$this->id'>";
    $output .= $embed_code;
    $output .= '</div>';

    return $output;
  }

  /**
   * Create markup for image element
   * @param type $variables
   * @return string
   */
  protected function formatImage($variables) {
    
    $feedback_options = get_option('enquir3_feedback_options_new', array());
      
    $src = $variables['src'];
    $href = $feedback_options['remote_url'] . "/images/$src";
    $alt = $variables['alt'];
    $class = $variables['class'];

    $output = "<div class='field-wrapper $class'>";
    $output .= "<img src='$href' alt='$alt' />";
    $output .= '</div>';

    return $output;
  }

  /**
   * Create markup for rating stars
   * @param type $variables
   * @return string
   */
  protected function formatRatingStars($variables) {

    $stars = $variables['stars'];
    $rating = $variables['rating'];
    $rating_value_text = number_format(5 * ($rating / 100), 1);
    $rating_value_output = "<span class='rating-value-number'>$rating_value_text</span>";
    $rating_value_output .= "<span class='rating-value-seperator'></span>";
    $rating_value_output .= "<span class='rating-value-total'>5</span>";

    $output = '<div class="rating-widget-line">';

    $output .= '<div class="rating-widget-label rating-widget-label-' . $variables['label'] . '"></div>';

    $output .= '<div class="rating-widget-stars-wrapper"><div class="rating-widget-stars">';
    for ($n = 1; $n <= $stars; $n++) {
      $star_value = ceil((100 / $stars) * $n);
      $prev_star_value = ceil((100 / $stars) * ($n - 1));
      $zebra = ($n % 2 == 0) ? 'even' : 'odd';
      $first = $n == 1 ? ' star-first' : '';
      $last = $n == $stars ? ' star-last' : '';
      $output .= '<div class="star star-' . $n . ' star-' . $zebra . $first . $last . '">';
      if ($rating < $star_value && $rating > $prev_star_value) {
        $percent = (($rating - $prev_star_value) / ($star_value - $prev_star_value)) * 100;
        $output .= '<span class="on" style="width: ' . $percent . '%">';
      }
      elseif ($rating >= $star_value) {
        $output .= '<span class="on">';
      }
      else {
        $output .= '<span class="off">';
      }
      $output .= '</span></div>';
    }
    $output .= '</div>';
    $output .= '<div class="rating-widget-value">' . $rating_value_output . '</div></div>';
    $output .= '</div>';

    return $output;
  }

}

/**
 * Dummy class to allow calling element create methods.
 */
class remstoItemHelper extends RemstoItem {
  
  public function __construct() {
    parent::__construct(NULL);
  }
  
  public function formatImage($variables) {
    return parent::formatImage($variables);
  }

}
